Package: mqqcause
Type: Package
Title: Multivariate Quantile-on-Quantile Granger Causality
Version: 1.0.0
Authors@R: c(
    person("Merwan", "Roudane",
           email = "merwanroudane920@gmail.com",
           role = c("aut", "cre", "cph")),
    person("Avik", "Sinha", role = "ctb",
           comment = "Original multivariate QQ causality methodology"),
    person("Nicholas", "Sim", role = "ctb",
           comment = "Original QQ regression methodology"),
    person("Hongtao", "Zhou", role = "ctb",
           comment = "Original QQ regression methodology"))
Description: Implements bivariate and Multivariate Quantile-on-Quantile
    Granger causality tests building on the Quantile-on-Quantile
    regression framework of Sim and Zhou (2015)
    <doi:10.1016/j.jbankfin.2015.01.013> and the quantile Granger
    causality test of Troster (2018)
    <doi:10.1080/07474938.2016.1172400>. The bivariate test estimates
    the local-linear slope in the quantile regression of y_t on lagged
    x_t with lagged y_t as control, using Gaussian kernel weights, and
    tests it against zero by paired bootstrap. The multivariate
    (conditional) test additionally conditions on a set of moderators Z
    and optional x times Z interaction terms, in the spirit of Sinha,
    Ghosh, Hussain, Nguyen and Das (2023)
    <doi:10.1016/j.eneco.2023.107021>. A Sup-Wald summary across the
    quantile grid is also provided. Heatmaps and 3D surfaces default to
    the 'MATLAB' 'Parula' colour map.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.1
Depends: R (>= 3.5.0)
Imports: quantreg (>= 5.0), plotly (>= 4.0.0), stats, utils, grDevices
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
URL: https://github.com/merwanroudane/qqcaus
BugReports: https://github.com/merwanroudane/qqcaus/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-28 13:02:20 UTC; HP
Author: Merwan Roudane [aut, cre, cph],
  Avik Sinha [ctb] (Original multivariate QQ causality methodology),
  Nicholas Sim [ctb] (Original QQ regression methodology),
  Hongtao Zhou [ctb] (Original QQ regression methodology)
Maintainer: Merwan Roudane <merwanroudane920@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-01 09:00:31 UTC
Built: R 4.4.3; ; 2026-06-01 12:35:14 UTC; unix

S3method(print, qq_causality)
S3method(print, mqq_causality)
S3method(summary, qq_causality)
S3method(summary, mqq_causality)
S3method(plot, qq_causality)
S3method(plot, mqq_causality)
export(qq_causality)
export(mqq_causality)
export(qq_causality_to_matrix)
export(sup_wald)
export(plot_qq_causality_3d)
export(plot_qq_causality_heatmap)
export(plot_qq_causality_contour)
export(plot_significance_heatmap)
export(qq_weights)
export(gaussian_kernel)
export(weighted_qr)
export(parula_colors)
export(matlab_jet_colors)
export(turbo_colors)
export(bluered_colors)
export(sinha_colors)
export(mqqcause_palette)
export(mqqcause_colorscales)
export(resolve_colorscale)
importFrom(grDevices, colorRampPalette)
importFrom(grDevices, rgb)
importFrom(plotly, plot_ly)
importFrom(plotly, layout)
importFrom(plotly, "%>%")
importFrom(quantreg, rq.wfit)
importFrom(stats, complete.cases)
importFrom(stats, dnorm)
importFrom(stats, median)
importFrom(stats, pnorm)
importFrom(stats, pt)
importFrom(stats, quantile)
importFrom(stats, sd)
importFrom(stats, var)
importFrom(utils, write.csv)

.onAttach <- function(libname, pkgname) {
  packageStartupMessage(
    "mqqcause 1.0.0  Multivariate QQ Granger Causality\n",
    "  Default colour scale: MATLAB Parula. See mqqcause_colorscales()."
  )
}

#' @title 3D Surface Plot of QQ Causality
#' @param qq_result A \code{qq_causality} / \code{mqq_causality} object.
#' @param value Column to plot. Default \code{"t_value"}.
#' @param colorscale Default \code{"Parula"}.
#' @param show_contour Show gridlines on the surface.
#' @param x_label,y_label,title Labels.
#' @return A plotly object.
#' @export
#' @importFrom plotly plot_ly layout "%>%"
plot_qq_causality_3d <- function(qq_result, value = "t_value",
                                 colorscale = "Parula", show_contour = TRUE,
                                 x_label = "X Quantile (tau)",
                                 y_label = "Y Quantile (theta)",
                                 title = NULL) {
  M <- qq_causality_to_matrix(qq_result, value)
  xs <- as.numeric(colnames(M)); ys <- as.numeric(rownames(M))
  zlab <- switch(value, t_value = "t-statistic",
                 p_value = "p-value",
                 beta1 = "beta1",
                 se = "SE", value)
  if (is.null(title))
    title <- paste(qq_result$direction, "- QQ Causality (", zlab, ")")
  cs <- resolve_colorscale(colorscale)
  step_x <- if (length(xs) > 1) diff(xs)[1] else 0.05
  step_y <- if (length(ys) > 1) diff(ys)[1] else 0.05
  plotly::plot_ly(
    x = xs, y = ys, z = M, type = "surface",
    colorscale = cs, showscale = TRUE,
    colorbar = list(title = zlab, tickformat = ".3f"),
    contours = list(
      x = list(show = show_contour, color = "black",
               start = min(xs), end = max(xs), size = step_x),
      y = list(show = show_contour, color = "black",
               start = min(ys), end = max(ys), size = step_y),
      z = list(show = FALSE)),
    lighting = list(ambient = 0.55, diffuse = 0.8,
                    specular = 0.15, roughness = 0.9)
  ) %>%
    plotly::layout(title = title,
                   paper_bgcolor = "white", plot_bgcolor = "white",
                   scene = list(
                     xaxis = list(title = x_label, tickformat = ".2f",
                                  showgrid = TRUE, gridcolor = "#E6E6E6"),
                     yaxis = list(title = y_label, tickformat = ".2f",
                                  showgrid = TRUE, gridcolor = "#E6E6E6"),
                     zaxis = list(title = zlab, showgrid = TRUE,
                                  gridcolor = "#F0F0F0"),
                     aspectratio = list(x = 1, y = 1, z = 0.7),
                     camera = list(eye = list(x = 1.4, y = 1.7, z = 1.2))))
}


#' @title Heatmap of QQ Causality with Significance Stars
#' @param qq_result A \code{qq_causality} / \code{mqq_causality} object.
#' @param value Column to plot. Default \code{"t_value"}.
#' @param colorscale Default \code{"Parula"}.
#' @param show_stars Overlay ***, **, * markers (p-value thresholds).
#' @param x_label,y_label,title Labels.
#' @return A plotly object.
#' @export
plot_qq_causality_heatmap <- function(qq_result, value = "t_value",
                                      colorscale = "Parula",
                                      show_stars = TRUE,
                                      x_label = "X Quantile (tau)",
                                      y_label = "Y Quantile (theta)",
                                      title = NULL) {
  M <- qq_causality_to_matrix(qq_result, value)
  xs <- as.numeric(colnames(M)); ys <- as.numeric(rownames(M))
  zlab <- switch(value, t_value = "t-statistic",
                 p_value = "p-value", beta1 = "beta1",
                 se = "SE", value)
  if (is.null(title))
    title <- paste(qq_result$direction, "- QQ Causality (", zlab, ")")
  cs <- resolve_colorscale(colorscale)
  p <- plotly::plot_ly(x = xs, y = ys, z = M, type = "heatmap",
                       colorscale = cs, showscale = TRUE,
                       hovertemplate = paste0("tau: %{x:.2f}<br>theta: %{y:.2f}<br>",
                                              zlab, ": %{z:.4f}<extra></extra>")) %>%
    plotly::layout(title = title,
                   xaxis = list(title = x_label),
                   yaxis = list(title = y_label))
  if (show_stars) {
    pmat <- qq_causality_to_matrix(qq_result, "p_value")
    stars <- matrix("", nrow = nrow(pmat), ncol = ncol(pmat))
    stars[pmat < 0.10] <- "*"; stars[pmat < 0.05] <- "**"; stars[pmat < 0.01] <- "***"
    anns <- list()
    for (i in seq_len(nrow(M))) for (j in seq_len(ncol(M))) {
      if (nzchar(stars[i, j])) {
        anns[[length(anns) + 1]] <- list(
          x = xs[j], y = ys[i], text = stars[i, j],
          xref = "x", yref = "y", showarrow = FALSE,
          font = list(size = 10, color = "white"))
      }
    }
    p <- p %>% plotly::layout(annotations = anns)
  }
  p
}


#' @title Contour Plot of QQ Causality
#' @param qq_result A \code{qq_causality} / \code{mqq_causality} object.
#' @param value,colorscale,x_label,y_label,title See
#'   \code{\link{plot_qq_causality_3d}}.
#' @return A plotly object.
#' @export
plot_qq_causality_contour <- function(qq_result, value = "t_value",
                                      colorscale = "Parula",
                                      x_label = "X Quantile (tau)",
                                      y_label = "Y Quantile (theta)",
                                      title = NULL) {
  M <- qq_causality_to_matrix(qq_result, value)
  xs <- as.numeric(colnames(M)); ys <- as.numeric(rownames(M))
  zlab <- switch(value, t_value = "t-statistic",
                 p_value = "p-value", beta1 = "beta1", se = "SE", value)
  if (is.null(title))
    title <- paste(qq_result$direction, "- QQ Causality (", zlab, ")")
  cs <- resolve_colorscale(colorscale)
  plotly::plot_ly(x = xs, y = ys, z = M, type = "contour",
                  colorscale = cs, showscale = TRUE,
                  contours = list(showlabels = TRUE)) %>%
    plotly::layout(title = title,
                   xaxis = list(title = x_label),
                   yaxis = list(title = y_label))
}


#' @title Significance-only heatmap (categorical)
#' @description Renders only the p-value significance categories
#'   (>=0.10, <0.10, <0.05, <0.01) so the causality region stands out.
#' @param qq_result A \code{qq_causality} / \code{mqq_causality} object.
#' @param colorscale Default \code{"Parula"}.
#' @return A plotly object.
#' @export
plot_significance_heatmap <- function(qq_result, colorscale = "Parula") {
  pmat <- qq_causality_to_matrix(qq_result, "p_value")
  sig <- matrix(0, nrow(pmat), ncol(pmat))
  sig[pmat < 0.10] <- 1; sig[pmat < 0.05] <- 2; sig[pmat < 0.01] <- 3
  xs <- as.numeric(colnames(pmat)); ys <- as.numeric(rownames(pmat))
  cs <- resolve_colorscale(colorscale)
  plotly::plot_ly(x = xs, y = ys, z = sig, type = "heatmap",
                  colorscale = cs, showscale = TRUE,
                  zmin = 0, zmax = 3,
                  colorbar = list(
                    title = "sig",
                    tickvals = c(0, 1, 2, 3),
                    ticktext = c("n.s.", "*", "**", "***"))) %>%
    plotly::layout(title = paste(qq_result$direction,
                                 "- significance (p < 0.10/0.05/0.01)"),
                   xaxis = list(title = "X Quantile (tau)"),
                   yaxis = list(title = "Y Quantile (theta)"))
}


#' @title S3 plot method for qq_causality
#' @param x A qq_causality / mqq_causality object.
#' @param value Column to plot.
#' @param colorscale Default \code{"Parula"}.
#' @param kind \code{"heatmap"}, \code{"3d"}, \code{"contour"}, or
#'   \code{"significance"}.
#' @param ... Passed to the underlying plot function.
#' @return A plotly object.
#' @export
plot.qq_causality <- function(x, value = "t_value", colorscale = "Parula",
                              kind = c("heatmap", "3d", "contour",
                                       "significance"), ...) {
  kind <- match.arg(kind)
  fn <- switch(kind,
               heatmap = plot_qq_causality_heatmap,
               "3d" = plot_qq_causality_3d,
               contour = plot_qq_causality_contour,
               significance = function(r, value, colorscale, ...)
                 plot_significance_heatmap(r, colorscale))
  fn(x, value = value, colorscale = colorscale, ...)
}


#' @title S3 plot method for mqq_causality
#' @param x An mqq_causality object.
#' @param value,colorscale,kind,... See \code{\link{plot.qq_causality}}.
#' @return A plotly object.
#' @export
plot.mqq_causality <- function(x, value = "t_value", colorscale = "Parula",
                               kind = c("heatmap", "3d", "contour",
                                        "significance"), ...) {
  NextMethod()
}

#' @title MATLAB Parula Color Palette
#' @description MATLAB R2014b Parula colormap as hex codes.
#' @param n Integer. Number of colours.
#' @return Character vector of hex colours.
#' @export
#' @importFrom grDevices colorRampPalette rgb
parula_colors <- function(n = 256) {
  stops <- rbind(
    c(0.2422, 0.1504, 0.6603), c(0.2504, 0.1650, 0.7076),
    c(0.2578, 0.1818, 0.7511), c(0.2647, 0.1978, 0.7952),
    c(0.2706, 0.2147, 0.8364), c(0.2751, 0.2342, 0.8710),
    c(0.2783, 0.2559, 0.8991), c(0.2803, 0.2782, 0.9221),
    c(0.2813, 0.3006, 0.9414), c(0.2810, 0.3228, 0.9579),
    c(0.2795, 0.3447, 0.9717), c(0.2760, 0.3667, 0.9829),
    c(0.2699, 0.3892, 0.9906), c(0.2602, 0.4123, 0.9952),
    c(0.2440, 0.4358, 0.9988), c(0.2206, 0.4603, 0.9973),
    c(0.1963, 0.4847, 0.9892), c(0.1834, 0.5074, 0.9798),
    c(0.1786, 0.5289, 0.9682), c(0.1764, 0.5499, 0.9520),
    c(0.1687, 0.5703, 0.9359), c(0.1540, 0.5902, 0.9218),
    c(0.1460, 0.6091, 0.9079), c(0.1380, 0.6276, 0.8973),
    c(0.1248, 0.6459, 0.8883), c(0.1113, 0.6635, 0.8763),
    c(0.0952, 0.6798, 0.8598), c(0.0689, 0.6948, 0.8394),
    c(0.0297, 0.7082, 0.8163), c(0.0036, 0.7203, 0.7917),
    c(0.0067, 0.7312, 0.7660), c(0.0433, 0.7411, 0.7394),
    c(0.0964, 0.7500, 0.7120), c(0.1408, 0.7584, 0.6842),
    c(0.1717, 0.7670, 0.6554), c(0.1938, 0.7758, 0.6251),
    c(0.2161, 0.7843, 0.5923), c(0.2470, 0.7918, 0.5567),
    c(0.2906, 0.7973, 0.5188), c(0.3406, 0.8008, 0.4789),
    c(0.3909, 0.8029, 0.4354), c(0.4456, 0.8024, 0.3909),
    c(0.5044, 0.7993, 0.3480), c(0.5616, 0.7942, 0.3045),
    c(0.6174, 0.7876, 0.2612), c(0.6720, 0.7793, 0.2227),
    c(0.7242, 0.7698, 0.1910), c(0.7738, 0.7598, 0.1646),
    c(0.8203, 0.7498, 0.1535), c(0.8634, 0.7406, 0.1596),
    c(0.9035, 0.7330, 0.1774), c(0.9393, 0.7288, 0.2100),
    c(0.9728, 0.7298, 0.2394), c(0.9956, 0.7434, 0.2371),
    c(0.9970, 0.7659, 0.2199), c(0.9952, 0.7893, 0.2028),
    c(0.9892, 0.8129, 0.1885), c(0.9786, 0.8360, 0.1766),
    c(0.9676, 0.8587, 0.1643), c(0.9610, 0.8806, 0.1537),
    c(0.9597, 0.9023, 0.1423), c(0.9628, 0.9234, 0.1330),
    c(0.9691, 0.9438, 0.1241), c(0.9769, 0.9839, 0.0805))
  hex <- grDevices::rgb(stops[, 1], stops[, 2], stops[, 3])
  if (n == nrow(stops)) return(hex)
  grDevices::colorRampPalette(hex)(n)
}


#' @title MATLAB Jet palette
#' @param n Integer. Number of colours.
#' @return Character vector of hex colours.
#' @export
matlab_jet_colors <- function(n = 256) {
  grDevices::colorRampPalette(
    c("#000090", "#0000FF", "#00FFFF",
      "#FFFF00", "#FF0000", "#800000"))(n)
}


#' @title Turbo palette
#' @param n Integer.
#' @return Character vector of hex colours.
#' @export
turbo_colors <- function(n = 256) {
  grDevices::colorRampPalette(
    c("#30123B", "#4145AB", "#4675ED", "#39A2FC", "#1BCFD4",
      "#24EAA7", "#61FC6C", "#A4FC3C", "#D1E834", "#F3C63A",
      "#FE9B2D", "#F36315", "#D93806", "#A91201", "#7A0403"))(n)
}


#' @title Blue-Red diverging palette
#' @param n Integer.
#' @return Character vector of hex colours.
#' @export
bluered_colors <- function(n = 256) {
  grDevices::colorRampPalette(
    c("#053061", "#2166AC", "#92C5DE", "#F7F7F7",
      "#F4A582", "#B2182B", "#67001F"))(n)
}


#' @title Sinha red-yellow-black palette
#' @param n Integer.
#' @return Character vector of hex colours.
#' @export
sinha_colors <- function(n = 256) {
  grDevices::colorRampPalette(
    c("#000000", "#4A0E00", "#B22222", "#FF8C00", "#FFFF66"))(n)
}


#' @title Convert R colour vector to a plotly colorscale list
#' @param cols Character vector of hex colours.
#' @param n_breaks Integer.
#' @return List of list(value, hex) pairs.
#' @export
mqqcause_palette <- function(cols, n_breaks = 32) {
  n_breaks <- max(2L, as.integer(n_breaks))
  idx <- round(seq(1, length(cols), length.out = n_breaks))
  vals <- seq(0, 1, length.out = n_breaks)
  lapply(seq_len(n_breaks), function(i) list(vals[i], cols[idx[i]]))
}


#' @title Resolve a colour-scale name to a plotly colorscale
#' @param name Character.
#' @param n_breaks Integer.
#' @return A plotly colorscale (character or list).
#' @export
resolve_colorscale <- function(name = "Parula", n_breaks = 32) {
  if (is.list(name)) return(name)
  if (!is.character(name) || length(name) != 1)
    stop("'name' must be a single string")
  key <- tolower(name)
  native <- c("viridis", "plasma", "cividis", "inferno", "magma",
              "rdbu", "rdylbu", "blues", "greens", "reds", "greys",
              "ylgnbu", "ylorrd", "bluered", "rainbow")
  if (key == "parula")  return(mqqcause_palette(parula_colors(), n_breaks))
  if (key == "jet")     return(mqqcause_palette(matlab_jet_colors(), n_breaks))
  if (key == "turbo")   return(mqqcause_palette(turbo_colors(), n_breaks))
  if (key == "bluered" || key == "blue_red")
    return(mqqcause_palette(bluered_colors(), n_breaks))
  if (key == "sinha" || key == "red_yellow_black")
    return(mqqcause_palette(sinha_colors(), n_breaks))
  if (key %in% native) {
    cap <- function(s) paste0(toupper(substr(s, 1, 1)), substr(s, 2, nchar(s)))
    return(cap(key))
  }
  name
}


#' @title Available colour scales in mqqcause
#' @param show_preview Print descriptions.
#' @return Character vector of scale names.
#' @export
mqqcause_colorscales <- function(show_preview = TRUE) {
  scales <- c("Parula", "Jet", "Turbo", "BlueRed", "Sinha",
              "Viridis", "Plasma", "Cividis", "Inferno", "Magma", "RdBu")
  desc <- c(Parula  = "MATLAB R2014b default (perceptually uniform)",
            Jet     = "Classic MATLAB rainbow",
            Turbo   = "Google Turbo",
            BlueRed = "Diverging (blue low, red high)",
            Sinha   = "Sinha cross-quantile heatmap palette",
            Viridis = "Perceptually uniform",
            Plasma  = "Perceptually uniform, high contrast",
            Cividis = "Optimized for CVD",
            Inferno = "Dark to bright",
            Magma   = "Dark purple to white",
            RdBu    = "Diverging red/blue")
  if (show_preview) {
    cat("\nAvailable mqqcause color scales\n")
    cat("===============================\n")
    for (s in scales) cat(sprintf("  %-8s : %s\n", s, desc[[s]]))
    cat("\n  Default in mqqcause plots: \"Parula\"\n")
  }
  invisible(scales)
}

#' @title Empirical CDF (mid-rank)
#' @keywords internal
empirical_cdf <- function(x) {
  ranks <- rank(as.numeric(x), ties.method = "average")
  ranks / (length(x) + 1)
}


#' @title Standard Gaussian Kernel
#' @param u Numeric vector.
#' @return Numeric vector of kernel weights.
#' @export
#' @importFrom stats dnorm
gaussian_kernel <- function(u) stats::dnorm(u)


#' @title QQ Kernel Weights on the Empirical-CDF Scale
#' @param x Numeric vector.
#' @param tau Numeric quantile in (0, 1).
#' @param h Numeric bandwidth.
#' @param cdf_based Logical. Use CDF-distance kernel.
#' @return Numeric vector of weights summing to \code{length(x)}.
#' @export
#' @importFrom stats sd
qq_weights <- function(x, tau, h = 0.05, cdf_based = TRUE) {
  x <- as.numeric(x)
  if (cdf_based) {
    Fn <- empirical_cdf(x); u <- (Fn - tau) / h
  } else {
    x_tau <- stats::quantile(x, tau, names = FALSE, na.rm = TRUE)
    s <- stats::sd(x, na.rm = TRUE)
    u <- (x - x_tau) / (if (s > 0) h * s else h)
  }
  w <- gaussian_kernel(u)
  if (sum(w) > 0) w <- w * (length(x) / sum(w))
  w
}


#' @title Weighted Quantile Regression
#' @param y Numeric vector.
#' @param X Numeric design matrix (intercept included).
#' @param tau Numeric quantile in (0, 1).
#' @param weights Numeric weights or NULL.
#' @return A list with \code{coef}, \code{residuals}, \code{fitted},
#'   \code{success}.
#' @export
#' @importFrom quantreg rq.wfit
weighted_qr <- function(y, X, tau, weights = NULL) {
  y <- as.numeric(y); X <- as.matrix(X); n <- nrow(X)
  if (is.null(weights)) weights <- rep(1, n)
  weights <- pmax(as.numeric(weights), 0)
  out <- tryCatch(
    quantreg::rq.wfit(X, y, tau = tau, weights = weights, method = "br"),
    error = function(e) NULL)
  if (is.null(out) || any(!is.finite(out$coefficients))) {
    return(list(coef = rep(NA_real_, ncol(X)),
                residuals = rep(NA_real_, n),
                fitted = rep(NA_real_, n),
                success = FALSE))
  }
  fitted <- as.numeric(X %*% out$coefficients)
  list(coef = unname(out$coefficients),
       residuals = as.numeric(y - fitted),
       fitted = fitted, success = TRUE)
}


#' @title Bootstrap SE for Weighted QR
#' @keywords internal
boot_wqr_se <- function(y, X, tau, weights, n_boot = 200, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  n <- length(y); k <- ncol(X)
  coefs <- matrix(NA_real_, n_boot, k)
  for (b in seq_len(n_boot)) {
    idx <- sample.int(n, n, replace = TRUE)
    fit <- weighted_qr(y[idx], X[idx, , drop = FALSE], tau, weights[idx])
    if (fit$success) coefs[b, ] <- fit$coef
  }
  se <- apply(coefs, 2, function(z) {
    z <- z[is.finite(z)]
    if (length(z) > 1) stats::sd(z) else NA_real_
  })
  list(se = se, boot = coefs)
}

#' @title Bivariate Quantile-on-Quantile Granger Causality
#'
#' @description
#' Tests whether the \eqn{\tau}-quantile of \eqn{x_{t-1}} Granger-causes
#' the \eqn{\theta}-quantile of \eqn{y_t} by inspecting the slope
#' \eqn{\beta_1(\theta,\tau)} in
#' \deqn{y_t = \beta_0(\theta,\tau) + \beta_1(\theta,\tau)(x_{t-1} - x^\tau)
#'        + \alpha(\theta) y_{t-1} + v_t^\theta,}
#' fitted by weighted quantile regression with Gaussian kernel weights.
#' Standard errors are computed by paired bootstrap. The function returns
#' a 2-D (theta, tau) surface of bootstrap t-statistics and p-values that
#' is ideal for a heatmap with significance stars (the standard
#' presentation used in Sinha et al. articles).
#'
#' @param x Numeric vector. Cause variable.
#' @param y Numeric vector. Effect variable.
#' @param y_quantiles Numeric vector of \eqn{\theta} in (0, 1).
#' @param x_quantiles Numeric vector of \eqn{\tau} in (0, 1).
#' @param bandwidth Kernel bandwidth on the empirical-CDF scale.
#' @param n_boot Bootstrap replicates. Default 200.
#' @param cdf_based_kernel Use CDF-distance kernel.
#' @param cause_name,effect_name Variable names.
#' @param verbose Print progress.
#' @param seed RNG seed.
#'
#' @return Object of class \code{"qq_causality"}.
#' @references
#' Sim, N., Zhou, H. (2015). \emph{J. Banking & Finance}, 55, 1-12.
#'
#' Troster, V. (2018). Testing for Granger-causality in Quantiles.
#' \emph{Econometric Reviews}, 37(8), 850-866.
#'
#' @examples
#' set.seed(1); n <- 200
#' x <- rnorm(n); y <- 0.3 * c(0, x[-n]) + rnorm(n, sd = 0.5)
#' fit <- qq_causality(x, y,
#'                     y_quantiles = c(0.25, 0.5, 0.75),
#'                     x_quantiles = c(0.25, 0.5, 0.75),
#'                     n_boot = 30, verbose = FALSE)
#' print(fit)
#'
#' @export
#' @importFrom stats quantile pt
qq_causality <- function(x, y,
                         y_quantiles = seq(0.05, 0.95, by = 0.05),
                         x_quantiles = seq(0.05, 0.95, by = 0.05),
                         bandwidth = 0.05,
                         n_boot = 200,
                         cdf_based_kernel = TRUE,
                         cause_name = "X",
                         effect_name = "Y",
                         verbose = TRUE,
                         seed = 42) {
  x <- as.numeric(x); y <- as.numeric(y)
  if (length(x) != length(y)) stop("x and y must have equal length")
  ok <- is.finite(x) & is.finite(y)
  x <- x[ok]; y <- y[ok]; n <- length(y)
  if (n < 30) stop("need at least 30 observations after NA-removal")

  y_resp <- y[-1]; x_lag <- x[-n]; y_lag <- y[-n]
  n_eff <- length(y_resp)

  if (verbose) {
    message("QQ Granger Causality   ", cause_name, " ==> ", effect_name)
    message("  n = ", n_eff,
            ", Y q-grid = ", length(y_quantiles),
            ", X q-grid = ", length(x_quantiles),
            ", h = ", bandwidth)
  }

  records <- list()
  total <- length(y_quantiles) * length(x_quantiles)
  done <- 0; pct_marker <- max(1L, total %/% 10L)

  for (tau in x_quantiles) {
    x_tau <- as.numeric(stats::quantile(x_lag, tau, na.rm = TRUE))
    z_x <- x_lag - x_tau
    w <- qq_weights(x_lag, tau, h = bandwidth, cdf_based = cdf_based_kernel)
    X_mat <- cbind(1, z_x, y_lag)

    for (theta in y_quantiles) {
      done <- done + 1L
      rec <- list(y_quantile = round(theta, 4),
                  x_quantile = round(tau, 4),
                  beta1 = NA_real_, se = NA_real_,
                  t_value = NA_real_, p_value = NA_real_)
      fit <- weighted_qr(y_resp, X_mat, theta, w)
      if (fit$success) {
        rec$beta1 <- fit$coef[2]
        bs <- boot_wqr_se(y_resp, X_mat, theta, w, n_boot,
                          if (!is.null(seed)) seed + done else NULL)
        sb1 <- bs$se[2]
        if (is.finite(sb1) && sb1 > 0) {
          rec$se <- sb1
          rec$t_value <- rec$beta1 / sb1
          df_r <- max(1, n_eff - ncol(X_mat))
          rec$p_value <- 2 * stats::pt(-abs(rec$t_value), df = df_r)
        }
      }
      records[[length(records) + 1]] <- rec
      if (verbose && done %% pct_marker == 0)
        message("  Progress: ", round(100 * done / total), "%")
    }
  }
  if (verbose) message("  done.")
  df <- do.call(rbind, lapply(records, as.data.frame,
                              stringsAsFactors = FALSE))
  df <- df[order(df$y_quantile, df$x_quantile), ]
  rownames(df) <- NULL

  res <- list(results = df,
              y_quantiles = y_quantiles,
              x_quantiles = x_quantiles,
              n_obs = n_eff, bandwidth = bandwidth,
              cause_name = cause_name, effect_name = effect_name,
              direction = paste(cause_name, "->", effect_name),
              call = match.call(),
              method = "QQ Granger Causality")
  class(res) <- "qq_causality"
  res
}


#' @title Multivariate (Conditional) QQ Granger Causality
#'
#' @description
#' Conditional QQ Granger-causality test from \code{x} to \code{y}
#' controlling for a set of moderators / controls \code{Z}. Same null as
#' \code{\link{qq_causality}} but the regression also includes
#' \eqn{Z_{j,t-1}} (and optionally \eqn{x_{t-1} \cdot Z_{j,t-1}}
#' interactions).
#'
#' @inheritParams qq_causality
#' @param moderators Named list of numeric vectors (the conditioning set Z).
#' @param interactions Logical. Include x*Z cross-terms.
#'
#' @return Object of class \code{"mqq_causality"}.
#'
#' @examples
#' set.seed(1); n <- 200
#' x <- rnorm(n); z <- rnorm(n)
#' y <- 0.3 * c(0, x[-n]) + 0.2 * c(0, z[-n]) + rnorm(n, sd = 0.4)
#' fit <- mqq_causality(x, y, list(Z = z),
#'                      y_quantiles = c(0.25, 0.5, 0.75),
#'                      x_quantiles = c(0.25, 0.5, 0.75),
#'                      n_boot = 30, verbose = FALSE)
#' print(fit)
#'
#' @export
mqq_causality <- function(x, y,
                          moderators = list(),
                          y_quantiles = seq(0.05, 0.95, by = 0.05),
                          x_quantiles = seq(0.05, 0.95, by = 0.05),
                          bandwidth = 0.05,
                          n_boot = 200,
                          interactions = TRUE,
                          cdf_based_kernel = TRUE,
                          cause_name = "X",
                          effect_name = "Y",
                          verbose = TRUE,
                          seed = 42) {
  x <- as.numeric(x); y <- as.numeric(y)
  if (length(x) != length(y)) stop("x and y must have equal length")
  if (!is.list(moderators) || (length(moderators) > 0 && is.null(names(moderators))))
    stop("'moderators' must be a named list of numeric vectors")

  Z_names <- names(moderators)
  if (length(Z_names) > 0) {
    Z <- do.call(cbind, lapply(seq_along(moderators), function(j) {
      a <- as.numeric(moderators[[j]])
      if (length(a) != length(y))
        stop("moderator '", Z_names[j], "' length mismatch")
      a
    }))
  } else {
    Z <- matrix(numeric(0), length(y), 0)
  }

  ok <- is.finite(x) & is.finite(y)
  if (ncol(Z) > 0) ok <- ok & apply(Z, 1, function(r) all(is.finite(r)))
  x <- x[ok]; y <- y[ok]; Z <- Z[ok, , drop = FALSE]
  n <- length(y)
  if (n < 40) stop("need at least 40 observations after NA-removal")

  y_resp <- y[-1]; x_lag <- x[-n]; y_lag <- y[-n]
  Z_lag <- Z[-n, , drop = FALSE]
  n_eff <- length(y_resp); p <- ncol(Z_lag)

  if (verbose) {
    message("Multivariate QQ Granger Causality   ",
            cause_name, " ==> ", effect_name, "  |  ",
            if (length(Z_names)) paste(Z_names, collapse = ",") else "none")
    message("  n = ", n_eff,
            ", Y q-grid = ", length(y_quantiles),
            ", X q-grid = ", length(x_quantiles),
            ", h = ", bandwidth,
            ", interactions = ", interactions)
  }

  records <- list()
  total <- length(y_quantiles) * length(x_quantiles)
  done <- 0; pct_marker <- max(1L, total %/% 10L)

  for (tau in x_quantiles) {
    x_tau <- as.numeric(stats::quantile(x_lag, tau, na.rm = TRUE))
    z_x <- x_lag - x_tau
    w <- qq_weights(x_lag, tau, h = bandwidth, cdf_based = cdf_based_kernel)

    Z_tau <- if (p > 0)
      vapply(seq_len(p),
             function(j) as.numeric(stats::quantile(Z_lag[, j], tau, na.rm = TRUE)),
             numeric(1))
      else numeric(0)
    z_Z <- if (p > 0) sweep(Z_lag, 2, Z_tau, "-") else matrix(numeric(0), n_eff, 0)
    inter <- if (interactions && p > 0)
      x_lag * Z_lag - matrix(rep(x_tau * Z_tau, each = n_eff), nrow = n_eff)
      else matrix(numeric(0), n_eff, 0)

    blocks <- list(rep(1, n_eff), z_x, y_lag)
    if (p > 0)           blocks[[length(blocks) + 1]] <- z_Z
    if (ncol(inter) > 0) blocks[[length(blocks) + 1]] <- inter
    X_mat <- do.call(cbind, blocks)

    for (theta in y_quantiles) {
      done <- done + 1L
      rec <- list(y_quantile = round(theta, 4),
                  x_quantile = round(tau, 4),
                  beta1 = NA_real_, se = NA_real_,
                  t_value = NA_real_, p_value = NA_real_)
      fit <- weighted_qr(y_resp, X_mat, theta, w)
      if (fit$success) {
        rec$beta1 <- fit$coef[2]
        bs <- boot_wqr_se(y_resp, X_mat, theta, w, n_boot,
                          if (!is.null(seed)) seed + done else NULL)
        sb1 <- bs$se[2]
        if (is.finite(sb1) && sb1 > 0) {
          rec$se <- sb1
          rec$t_value <- rec$beta1 / sb1
          df_r <- max(1, n_eff - ncol(X_mat))
          rec$p_value <- 2 * stats::pt(-abs(rec$t_value), df = df_r)
        }
      }
      records[[length(records) + 1]] <- rec
      if (verbose && done %% pct_marker == 0)
        message("  Progress: ", round(100 * done / total), "%")
    }
  }
  if (verbose) message("  done.")
  df <- do.call(rbind, lapply(records, as.data.frame,
                              stringsAsFactors = FALSE))
  df <- df[order(df$y_quantile, df$x_quantile), ]
  rownames(df) <- NULL

  res <- list(results = df,
              y_quantiles = y_quantiles,
              x_quantiles = x_quantiles,
              n_obs = n_eff, bandwidth = bandwidth,
              cause_name = cause_name, effect_name = effect_name,
              direction = paste(cause_name, "->", effect_name),
              moderator_names = Z_names,
              interactions = interactions,
              call = match.call(),
              method = "Multivariate QQ Granger Causality (conditional)")
  class(res) <- c("mqq_causality", "qq_causality")
  res
}


#' @title Sup-Wald Summary (Troster 2018)
#'
#' @description Reports \eqn{S = \sup_{(\theta,\tau)} |t(\theta,\tau)|}
#'   from a QQ causality result with a Bonferroni-adjusted p-value over
#'   the (theta, tau) grid.
#' @param qq_result A \code{qq_causality} or \code{mqq_causality} object.
#' @param alpha Significance level. Default 0.05.
#' @return A list with sup statistic, argmax and Bonferroni p-value.
#' @export
#' @importFrom stats pnorm
sup_wald <- function(qq_result, alpha = 0.05) {
  if (!inherits(qq_result, "qq_causality"))
    stop("'qq_result' must be a qq_causality / mqq_causality object")
  r <- qq_result$results
  r <- r[is.finite(r$t_value), ]
  if (!nrow(r))
    return(list(sup_statistic = NA, argmax_y_quantile = NA,
                argmax_x_quantile = NA, bonferroni_p_value = NA,
                reject_H0_at_alpha = NA))
  i_max <- which.max(abs(r$t_value))
  sup <- abs(r$t_value[i_max])
  pg <- 2 * stats::pnorm(-sup)
  adj <- min(1, pg * nrow(r))
  list(sup_statistic = sup,
       argmax_y_quantile = r$y_quantile[i_max],
       argmax_x_quantile = r$x_quantile[i_max],
       bonferroni_p_value = adj,
       reject_H0_at_alpha = adj < alpha)
}


#' @title Print method for qq_causality
#' @param x A qq_causality object.
#' @param ... Ignored.
#' @return Invisibly returns \code{x}.
#' @export
print.qq_causality <- function(x, ...) {
  cat("\n", x$method, "\n", sep = "")
  cat(strrep("=", 60), "\n", sep = "")
  cat("  Direction : ", x$direction, "\n", sep = "")
  cat("  N         : ", x$n_obs, "  h = ", x$bandwidth, "\n", sep = "")
  cat("  Q-grids   : Y = ", length(x$y_quantiles),
      "   X = ", length(x$x_quantiles), "\n", sep = "")
  r <- x$results
  rok <- r[is.finite(r$t_value), ]
  if (nrow(rok) > 0) {
    p <- rok$p_value
    cat(sprintf("\n  Cells tested : %d\n", nrow(rok)))
    cat(sprintf("    p<0.10 : %d (%0.1f%%)\n",
                sum(p < 0.10, na.rm = TRUE),
                100 * sum(p < 0.10, na.rm = TRUE) / nrow(rok)))
    cat(sprintf("    p<0.05 : %d (%0.1f%%)\n",
                sum(p < 0.05, na.rm = TRUE),
                100 * sum(p < 0.05, na.rm = TRUE) / nrow(rok)))
    cat(sprintf("    p<0.01 : %d (%0.1f%%)\n",
                sum(p < 0.01, na.rm = TRUE),
                100 * sum(p < 0.01, na.rm = TRUE) / nrow(rok)))
    sw <- sup_wald(x)
    cat(sprintf("\n  Sup-Wald (Troster 2018)\n"))
    cat(sprintf("    sup|t| = %0.4f  at (theta=%0.2f, tau=%0.2f)\n",
                sw$sup_statistic, sw$argmax_y_quantile,
                sw$argmax_x_quantile))
    cat(sprintf("    Bonferroni p = %0.4g    reject H0 at 5%% : %s\n",
                sw$bonferroni_p_value, sw$reject_H0_at_alpha))
  }
  invisible(x)
}


#' @title Print method for mqq_causality
#' @param x An mqq_causality object.
#' @param ... Ignored.
#' @return Invisibly returns \code{x}.
#' @export
print.mqq_causality <- function(x, ...) {
  NextMethod()
  if (length(x$moderator_names))
    cat("  Conditioning set : ",
        paste(x$moderator_names, collapse = ", "),
        " (interactions = ", x$interactions, ")\n", sep = "")
  invisible(x)
}


#' @title Summary methods
#' @param object A qq_causality / mqq_causality object.
#' @param ... Ignored.
#' @return Invisibly returns \code{object}.
#' @export
summary.qq_causality <- function(object, ...) { print(object); invisible(object) }

#' @rdname summary.qq_causality
#' @export
summary.mqq_causality <- function(object, ...) { print(object); invisible(object) }


#' @title Convert QQ-causality result to matrix
#' @param qq_result A qq_causality / mqq_causality object.
#' @param value Column to pivot: \code{"t_value"} (default),
#'   \code{"p_value"}, \code{"beta1"}, \code{"se"}.
#' @return Numeric matrix.
#' @export
qq_causality_to_matrix <- function(qq_result, value = "t_value") {
  if (!inherits(qq_result, "qq_causality"))
    stop("'qq_result' must be a qq_causality / mqq_causality object")
  df <- qq_result$results
  if (!value %in% names(df)) stop("value '", value, "' not in results")
  ys <- sort(unique(df$y_quantile))
  xs <- sort(unique(df$x_quantile))
  M <- matrix(NA_real_, length(ys), length(xs),
              dimnames = list(as.character(ys), as.character(xs)))
  for (i in seq_along(ys)) for (j in seq_along(xs)) {
    k <- which(df$y_quantile == ys[i] & df$x_quantile == xs[j])
    if (length(k)) M[i, j] <- df[[value]][k[1]]
  }
  M
}

test_that("parula returns hex colours", {
  expect_match(parula_colors(5), "^#[0-9A-Fa-f]{6}$")
})

test_that("qq_causality returns expected structure", {
  set.seed(1); n <- 100
  x <- rnorm(n); y <- 0.3 * c(0, x[-n]) + rnorm(n, sd = 0.5)
  fit <- qq_causality(x, y,
                      y_quantiles = c(0.25, 0.5, 0.75),
                      x_quantiles = c(0.25, 0.5, 0.75),
                      n_boot = 20, verbose = FALSE)
  expect_s3_class(fit, "qq_causality")
  expect_equal(nrow(fit$results), 9)
  M <- qq_causality_to_matrix(fit, "beta1")
  expect_equal(dim(M), c(3, 3))
  sw <- sup_wald(fit)
  expect_true(is.list(sw) && "sup_statistic" %in% names(sw))
})

test_that("mqq_causality conditions on Z", {
  set.seed(1); n <- 120
  x <- rnorm(n); z <- rnorm(n)
  y <- 0.3 * c(0, x[-n]) + 0.2 * c(0, z[-n]) + rnorm(n, sd = 0.4)
  fit <- mqq_causality(x, y, list(Z = z),
                       y_quantiles = c(0.25, 0.5, 0.75),
                       x_quantiles = c(0.25, 0.5, 0.75),
                       n_boot = 20, verbose = FALSE)
  expect_s3_class(fit, "mqq_causality")
  expect_true("Z" %in% fit$moderator_names)
})

#' @keywords internal
"_PACKAGE"

#' mqqcause: Multivariate Quantile-on-Quantile Granger Causality
#'
#' Implements bivariate Quantile-on-Quantile Granger causality and the
#' multivariate (conditional) version with a vector of moderators and
#' optional interaction terms. A Sup-Wald summary across the (theta, tau)
#' grid is also provided (Troster, 2018).
#'
#' @docType package
#' @name mqqcause-package
NULL

\name{plot_qq_causality_3d}
\alias{plot_qq_causality_3d}
\alias{plot_qq_causality_heatmap}
\alias{plot_qq_causality_contour}
\alias{plot_significance_heatmap}
\title{Visualisations for QQ Causality Results}
\usage{
plot_qq_causality_3d(qq_result, value = "t_value",
                     colorscale = "Parula", show_contour = TRUE,
                     x_label = "X Quantile (tau)",
                     y_label = "Y Quantile (theta)", title = NULL)

plot_qq_causality_heatmap(qq_result, value = "t_value",
                          colorscale = "Parula", show_stars = TRUE,
                          x_label = "X Quantile (tau)",
                          y_label = "Y Quantile (theta)", title = NULL)

plot_qq_causality_contour(qq_result, value = "t_value",
                          colorscale = "Parula",
                          x_label = "X Quantile (tau)",
                          y_label = "Y Quantile (theta)", title = NULL)

plot_significance_heatmap(qq_result, colorscale = "Parula")
}
\arguments{
\item{qq_result}{A \code{qq_causality} or \code{mqq_causality} object.}
\item{value}{Column to plot.}
\item{colorscale}{Default \code{"Parula"}.}
\item{show_contour, show_stars, x_label, y_label, title}{Plot options.}
}
\value{A plotly object.}
\description{3D surface, heatmap, contour and significance-only heatmap
visualisations of QQ Granger-causality results, defaulting to MATLAB Parula.}
\examples{
## Small toy example -- auto-tested.
set.seed(1); n <- 80
x <- rnorm(n); y <- 0.3 * c(0, x[-n]) + rnorm(n, sd = 0.4)
fit <- qq_causality(x, y,
                    y_quantiles = c(0.25, 0.5, 0.75),
                    x_quantiles = c(0.25, 0.5, 0.75),
                    n_boot = 10, verbose = FALSE)
p1 <- plot_qq_causality_heatmap(fit, value = "t_value", show_stars = TRUE)
p2 <- plot_qq_causality_3d(fit, value = "t_value")
p3 <- plot_qq_causality_contour(fit, value = "t_value")
p4 <- plot_significance_heatmap(fit)
}

\name{qq_causality}
\alias{qq_causality}
\alias{mqq_causality}
\alias{sup_wald}
\alias{qq_causality_to_matrix}
\title{Quantile-on-Quantile Granger Causality (Bivariate and Multivariate)}
\usage{
qq_causality(x, y,
             y_quantiles = seq(0.05, 0.95, by = 0.05),
             x_quantiles = seq(0.05, 0.95, by = 0.05),
             bandwidth = 0.05, n_boot = 200,
             cdf_based_kernel = TRUE,
             cause_name = "X", effect_name = "Y",
             verbose = TRUE, seed = 42)

mqq_causality(x, y, moderators = list(),
              y_quantiles = seq(0.05, 0.95, by = 0.05),
              x_quantiles = seq(0.05, 0.95, by = 0.05),
              bandwidth = 0.05, n_boot = 200, interactions = TRUE,
              cdf_based_kernel = TRUE,
              cause_name = "X", effect_name = "Y",
              verbose = TRUE, seed = 42)

sup_wald(qq_result, alpha = 0.05)
qq_causality_to_matrix(qq_result, value = "t_value")
}
\arguments{
\item{x, y}{Numeric vectors (cause and effect).}
\item{moderators}{Named list of numeric vectors (the conditioning set Z).}
\item{y_quantiles, x_quantiles}{Numeric vectors of quantile levels in (0, 1).}
\item{bandwidth}{Kernel bandwidth on the empirical-CDF scale.}
\item{n_boot}{Bootstrap replicates.}
\item{interactions}{Include x*Z cross-terms (mqq_causality).}
\item{cdf_based_kernel}{Use CDF-distance kernel.}
\item{cause_name, effect_name}{Variable names for printing.}
\item{verbose}{Print progress.}
\item{seed}{RNG seed.}
\item{qq_result}{A \code{qq_causality} / \code{mqq_causality} object.}
\item{value}{Column to pivot.}
\item{alpha}{Significance threshold.}
}
\value{For \code{qq_causality} and \code{mqq_causality}, an object of class
\code{"qq_causality"} or \code{"mqq_causality"}. \code{sup_wald} returns a
list; \code{qq_causality_to_matrix} returns a numeric matrix.}
\description{Tests whether the tau-quantile of \eqn{x_{t-1}} Granger-causes
the theta-quantile of \eqn{y_t}. The multivariate variant conditions on a
set of moderators \eqn{Z} and optional \eqn{x \cdot Z} interaction terms
(the conditional-causality test that under-pins Sinha et al., 2024).}
\references{
Sim, N., Zhou, H. (2015). \emph{Journal of Banking and Finance}, 55, 1-12.

Troster, V. (2018). Testing for Granger-causality in Quantiles.
\emph{Econometric Reviews}, 37(8), 850-866.
}
\examples{
set.seed(1); n <- 200
x <- rnorm(n); z <- rnorm(n)
y <- 0.3 * c(0, x[-n]) + 0.2 * c(0, z[-n]) + rnorm(n, sd = 0.4)
fit <- mqq_causality(x, y, list(Z = z),
                     y_quantiles = c(0.25, 0.5, 0.75),
                     x_quantiles = c(0.25, 0.5, 0.75),
                     n_boot = 30, verbose = FALSE)
print(fit)
sup_wald(fit)
}

\name{qq_weights}
\alias{qq_weights}
\alias{gaussian_kernel}
\alias{weighted_qr}
\title{Numerical building blocks}
\usage{
qq_weights(x, tau, h = 0.05, cdf_based = TRUE)
gaussian_kernel(u)
weighted_qr(y, X, tau, weights = NULL)
}
\arguments{
\item{x, y, u}{Numeric vectors.}
\item{X}{Numeric design matrix.}
\item{tau}{Quantile in (0, 1).}
\item{h}{Bandwidth.}
\item{cdf_based}{Use empirical-CDF distance kernel.}
\item{weights}{Optional numeric weights.}
}
\value{Numeric vector or list.}
\description{Gaussian kernel, QQ weights, and weighted quantile regression
via \code{quantreg::rq.wfit}. Exposed for advanced users.}
\examples{
set.seed(1)
x <- rnorm(50)
w <- qq_weights(x, tau = 0.5)
sum(w)            # weights sum to length(x)
k <- gaussian_kernel(seq(-3, 3, by = 1))
X <- cbind(1, x)
y <- 0.5 * x + rnorm(50, sd = 0.3)
fit <- weighted_qr(y, X, tau = 0.5, weights = w)
fit$coef
}

\name{parula_colors}
\alias{parula_colors}
\alias{matlab_jet_colors}
\alias{turbo_colors}
\alias{bluered_colors}
\alias{sinha_colors}
\alias{mqqcause_palette}
\alias{resolve_colorscale}
\alias{mqqcause_colorscales}
\title{MATLAB-style colour palettes for mqqcause}
\usage{
parula_colors(n = 256)
matlab_jet_colors(n = 256)
turbo_colors(n = 256)
bluered_colors(n = 256)
sinha_colors(n = 256)
mqqcause_palette(cols, n_breaks = 32)
resolve_colorscale(name = "Parula", n_breaks = 32)
mqqcause_colorscales(show_preview = TRUE)
}
\arguments{
\item{n}{Number of colours.}
\item{cols}{Character vector of hex colours.}
\item{n_breaks}{Stops for the plotly list.}
\item{name}{Scale name (Parula by default).}
\item{show_preview}{Print descriptions.}
}
\value{Character vector or list.}
\description{Colour palettes used by mqqcause plotting functions. The
default scale is MATLAB Parula.}
\examples{
parula_colors(8)
matlab_jet_colors(8)
turbo_colors(8)
bluered_colors(8)
sinha_colors(8)
mqqcause_colorscales(show_preview = FALSE)
}

\name{mqqcause-package}
\alias{mqqcause-package}
\docType{package}
\title{mqqcause: Multivariate Quantile-on-Quantile Granger Causality}
\description{Implements bivariate Quantile-on-Quantile Granger causality
and the multivariate (conditional) version with a vector of moderators
and optional interaction terms. A Sup-Wald summary across the
(theta, tau) grid is also provided (Troster, 2018).}
\author{Dr Merwan Roudane \email{merwanroudane920@gmail.com}\cr
  GitHub: \url{https://github.com/merwanroudane/qqcaus}}
\keyword{package}
